import sys
import os
import logging
from selenium import webdriver
from selenium.webdriver.firefox.service import Service
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.keys import Keys
import time

# Configure logging
log_path = "D:/MyPVD/PVD_0.9.9.21_MOD-Simple AllMovies/Scripts/python_script.log"
logging.basicConfig(
    filename=log_path,
    filemode='w',
    level=logging.INFO,
    format='%(asctime)s - %(levelname)s - %(message)s'
)

logging.info("Starting the Python script.")

# Check if the IMDb URL is passed as a parameter
if len(sys.argv) < 2:
    logging.error("IMDb URL was not provided as a parameter.")
    sys.exit("IMDb URL was not provided as a parameter.")

imdb_url = sys.argv[1]
logging.info(f"Using IMDb URL: {imdb_url}")

# Path to geckodriver.exe
gecko_path = "C:/Projects/geckodriver.exe"

# Define the PVD path
pvd_path = "D:/MyPVD/PVD_0.9.9.21_MOD-Simple AllMovies1"
output_path = os.path.join(pvd_path, "Scripts", "Tmp", "downpage-UTF8_NO_BOM.htm")

os.makedirs(os.path.dirname(output_path), exist_ok=True)
logging.info(f"Output path set to: {output_path}")

# Create a browser object
service = Service(gecko_path)
driver = webdriver.Firefox(service=service)

try:
    # Open the IMDb page
    driver.get(imdb_url)
    logging.info(f"Page {imdb_url} loaded successfully.")

    # Handle "Select Your Preferences" popup
    try:
        popup = WebDriverWait(driver, 10).until(
            EC.presence_of_element_located((By.XPATH, "//div[contains(@class, 'sc-kDvujY')]"))
        )
        logging.info("'Select Your Preferences' popup detected.")

        accept_button = WebDriverWait(driver, 10).until(
            EC.element_to_be_clickable((By.XPATH, "//button[@data-testid='accept-button']"))
        )
        accept_button.click()
        logging.info("Clicked on the 'Accept' button to close the popup.")
    except Exception as e:
        logging.info("No 'Select Your Preferences' popup appeared or an error occurred.")

    # Wait for the page to load
    time.sleep(5)

    # Improved function to click all "More" buttons with scrolling
    def click_all_more_buttons(driver):
        """
        Scrolls down the page and clicks all the "More" buttons that are visible.
        """
        body = driver.find_element(By.TAG_NAME, 'body')
        while True:
            try:
                # Find visible "More" buttons
                more_buttons = driver.find_elements(By.XPATH, '//span[contains(@class, "ipc-see-more__text")]/..')

                if not more_buttons:
                    logging.info("No more 'More' buttons found.")
                    break

                for button in more_buttons:
                    try:
                        # Scroll into view before clicking
                        driver.execute_script("arguments[0].scrollIntoView({block: 'center'});", button)
                        time.sleep(0.5)  # Allow page to stabilize
                        button.click()
                        logging.info("Clicked a 'More' button.")
                        time.sleep(1)  # Allow time for new content to load
                    except Exception as e:
                        logging.warning(f"Error clicking a 'More' button: {e}")
                        continue

                # Scroll the page down to load more buttons
                body.send_keys(Keys.PAGE_DOWN)
                time.sleep(1)  # Wait for page to load

            except Exception as e:
                logging.info("No additional 'More' buttons to click.")
                break

    # Execute the function
    click_all_more_buttons(driver)

    # Retrieve the full source HTML of the page after all "More" buttons are clicked
    html_source = driver.page_source

    # Save the HTML to a file
    with open(output_path, 'w', encoding='utf-8') as file:
        file.write(html_source)
    logging.info(f"HTML saved to file: {output_path}")

except Exception as e:
    logging.error(f"An error occurred: {e}")

finally:
    # Ensure the browser is closed even if an error occurs
    try:
        driver.quit()
        logging.info("Browser closed.")
    except Exception as e:
        logging.error(f"Error closing the browser: {e}")